/**
****************************************************************************
* @file         lock_simulation.c
* @brief        Lock simulation application. It sends a frame when button1 of the
*               evaluation board is pushed.
*
* @version      1.0
* @date         2019-03-07
****************************************************************************
* @author       Kepa Beaskoetxea Ibarra
* @copyright    Ojmar S.A.
****************************************************************************
* @par Version history
* @par
*  Version   | Date         |    Revised by    |  Description
*  --------- | ------------ | ---------------  | ----------------------------
*      1.0   | 2019-03-07   |     kbeasko      | Original version
****************************************************************************/

/*
 * $ Copyright Cypress Semiconductor $
 */

/** @file
 *
 * Hibernation test
 *
 */
 
/******************************************************
 *                     Includes
 ******************************************************/
#include "wiced.h"

/******************************************************
 *                      Macros
 ******************************************************/
#define HIBERNATION_PERIOD_MS 10000
//#define PLATFORM_HIB_ENABLE
/******************************************************
 *                    Constants
 ******************************************************/
#define TCP_PACKET_MAX_DATA_LENGTH        70
#define TCP_CLIENT_INTERVAL               2
#define TCP_SERVER_PORT                   50007 //15002
#define TCP_CLIENT_CONNECT_TIMEOUT        3000
#define TCP_CLIENT_RECEIVE_TIMEOUT        1000
#define TCP_CONNECTION_NUMBER_OF_RETRIES  3

/* Change the server IP address to match the real TCP server address */
#define TCP_SERVER_IP_ADDRESS MAKE_IPV4_ADDRESS(192,168,17,9)

/* The frames to send to TCP server */
#define FRAME_OK         "0005FFFE" //FFFE, 6666, free user card
#define FRAME_NOK        "0005FFFL" 
/******************************************************
 *                   Enumerations
 ******************************************************/
typedef enum {
    idle,    //low power state (default)
    send_data,
}STATES;
/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Function Declarations
 ******************************************************/
static wiced_result_t tcp_client();

/******************************************************
 *               Variable Definitions
 ******************************************************/
static wiced_tcp_socket_t  tcp_client_socket;
static wiced_result_t tcp_client(void);
static const wiced_ip_setting_t device_init_ip_settings =
{
    INITIALISER_IPV4_ADDRESS(.ip_address, MAKE_IPV4_ADDRESS(192,168, 17, 71)),
    INITIALISER_IPV4_ADDRESS(.netmask,    MAKE_IPV4_ADDRESS(255,255,255,  0)),
    INITIALISER_IPV4_ADDRESS(.gateway,    MAKE_IPV4_ADDRESS(192,168, 17,  1)),
};

/******************************************************
 *               Function Definitions
 ******************************************************/
void application_start() {
    STATES            state = idle;
    uint32_t          freq;
    uint32_t          max_ticks;
    uint32_t          ticks_to_wakeup;
    platform_result_t result = PLATFORM_BADARG;


    /* Initialise the device and WICED framework */
    wiced_init();

    /* Set the beacon listen interval */
    wiced_wifi_set_listen_interval( 10, WICED_LISTEN_INTERVAL_TIME_UNIT_BEACON );

    //printf("After short sleep will hibernate for %u ms\n", (unsigned)HIBERNATION_PERIOD_MS);
    //host_rtos_delay_milliseconds(1000);
    printf("\n======================================");
    printf("\n== Lock simulation test application ==");
    printf("\n======================================\n\n");
    printf("\nSetting up hibernation parameters...\n");

    /* Configure 'Hibernation' parameters */
    freq      = platform_hibernation_get_clock_freq();
    max_ticks = platform_hibernation_get_max_ticks();
    ticks_to_wakeup = freq * HIBERNATION_PERIOD_MS / 1000;
    printf("\tFrequency = %u \n\tMaximum ticks = 0x%x \n\tTicks to wakeup = %u \n\n\n", (unsigned)freq, (unsigned)max_ticks, (unsigned)ticks_to_wakeup);




    while(1){
        switch (state){
        case idle:
            /* Suspend networking activity while sleeping */
            wiced_network_suspend();
            /* Go to low power mode */
            printf("\nSe va a hibernacin...\n");
           // platform_hibernation_start(ticks_to_wakeup);

            /* For simulating purposes */
            if(wiced_gpio_input_get(WICED_BUTTON1) == 1){ 
                /* Lock operates and creates FRAME to be send */
                state = send_data;
            }
            break;
        case send_data:
            wiced_network_resume();
            tcp_client();
            state = idle;
            break;
        }
    }
}


static wiced_result_t tcp_client( void )
{
    uint32_t                 start_time, end_time;
    wiced_result_t           result;
    wiced_packet_t*          packet;
    wiced_packet_t*          rx_packet;
    char*                    tx_data;
    char*                    rx_data;
    uint16_t                 offset;
    uint16_t                 rx_data_length;
    uint16_t                 available_data_length;
    const wiced_ip_address_t INITIALISER_IPV4_ADDRESS(server_ip_address, TCP_SERVER_IP_ADDRESS);
    int                      host_connection_retries;

    /* Connect to the remote TCP server, try several times */
    host_connection_retries = 0;
    do
    {
        result = wiced_tcp_connect(&tcp_client_socket, &server_ip_address, TCP_SERVER_PORT, TCP_CLIENT_CONNECT_TIMEOUT);
        host_connection_retries++;
    }
    while((result != WICED_SUCCESS) && (host_connection_retries < TCP_CONNECTION_NUMBER_OF_RETRIES));
    if( result != WICED_SUCCESS)
    {
        WPRINT_APP_INFO(("Unable to connect to the server! Halt.\r\n"));
        return WICED_ERROR;
    }
    result = wiced_time_get_time(&start_time);

    /* Create the TCP packet. Memory for the tx_data is automatically allocated */
    if (wiced_packet_create_tcp(&tcp_client_socket, TCP_PACKET_MAX_DATA_LENGTH, &packet, (uint8_t**)&tx_data, &available_data_length) != WICED_SUCCESS)
    {
        WPRINT_APP_INFO(("TCP packet creation failed\r\n"));
        return WICED_ERROR;
    }

    /* Write the message into tx_data"  */
    //sprintf(tx_data, "%s", "Hello from WICED\r\n");
    sprintf(tx_data, "%s", FRAME_OK);

    /* Set the end of the data portion */
    wiced_packet_set_data_end(packet, (uint8_t*)tx_data + strlen(tx_data));

    /* Send the TCP packet */
    if (wiced_tcp_send_packet(&tcp_client_socket, packet) != WICED_SUCCESS)
    {
        WPRINT_APP_INFO(("TCP packet send failed\r\n"));

        /* Delete packet, since the send failed */
        wiced_packet_delete(packet);

        /* Close the connection */
        wiced_tcp_disconnect(&tcp_client_socket);
        return WICED_ERROR;
    }

    /* Receive a response from the server and print it out to the serial console */
    result = wiced_tcp_receive(&tcp_client_socket, &rx_packet, TCP_CLIENT_RECEIVE_TIMEOUT);
    if( result != WICED_SUCCESS )
    {
        WPRINT_APP_INFO(("TCP packet reception failed\r\n"));

        /* Delete packet, since the receive failed */
        wiced_packet_delete(rx_packet);

        /* Close the connection */
        wiced_tcp_disconnect(&tcp_client_socket);
        return WICED_ERROR;
    }
    /* Get the contents of the received packet */
    offset = 0;
    do
    {
        wiced_packet_get_data(rx_packet, offset, (uint8_t**)&rx_data, &rx_data_length, &available_data_length);

        offset += rx_data_length;
    } while (rx_data_length < available_data_length);

    /* Print the content of the received message */
    WPRINT_APP_INFO(("%s", rx_data));

    /* Delete the packet and terminate the connection */
    wiced_packet_delete(rx_packet);
    wiced_tcp_disconnect(&tcp_client_socket);
    result = wiced_time_get_time( &end_time );
    WPRINT_APP_INFO( ("Ping Successful - TimeMs: %d \r\n", (int)( end_time - start_time ) ) );


    return WICED_SUCCESS;

}




